
local map = require('gloves.map')
local component = require('gloves.component')


local platformer = component:new()

platformer.config_double_jump = false

platformer.speed_walk = 0.9
platformer.speed_run = 1.4
platformer.speed_jump = 6.25
platformer.speed_gravity = 0.35

platformer.friction_walk = 0.7
platformer.friction_jump_gravity = 0.6

platformer.on_ground_distance = 3

platformer.state_grounded = false
platformer.state_jumping = false
platformer.state_double_jumping = false
platformer.state_double_jump_ready = false

platformer.input_left = false
platformer.input_right = false
platformer.input_jump = false
platformer.input_jump_hold = false


function platformer:update()

  self.state_grounded = self.e.map:tile_below(self.e, self.on_ground_distance)

  if self.e.map:entity_below(self.e, self.on_ground_distance) then
    self.state_grounded = true
  end

  -- left and right
  if self.input_left then
    if self.input_run then
      self.e.xa = self.e.xa - self.speed_run
    else
      self.e.xa = self.e.xa - self.speed_walk
    end

    self.e.d = map.direction.L
  elseif self.input_right then
    if self.input_run then
      self.e.xa = self.e.xa + self.speed_run
    else
      self.e.xa = self.e.xa + self.speed_walk
    end

    self.e.d = map.direction.R
  end

  -- on ground
  if self.state_grounded then
    self.state_jumping = false
    self.state_double_jumping = false
    self.state_double_jump_ready = false
  end

  -- if jumping and jump key released
  -- if not already double jumping set ready
  if self.state_jumping and not self.input_jump_hold then
    if not self.state_double_jumping then
      self.state_double_jump_ready = true
    end

    self.state_jumping = false
    self.state_double_jumping = false
  end

  -- jump
  if self.state_grounded and self.input_jump then
    self.state_grounded = false
    self.state_jumping = true
    self.e.ya = self.e.ya - self.speed_jump
  end

  -- double jump
  if self.config_double_jump and self.state_double_jump_ready and self.input_jump then
    -- no longer ready to double
    self.state_double_jump_ready = false

    -- jumping and double jumping
    self.state_jumping = true
    self.state_double_jumping = true
    self.e.ya = self.e.ya - self.speed_jump
  end

  -- actual change
  self.e.dx = self.e.xa
  self.e.dy = self.e.ya

  -- actual move
  self.e.map:move_entity(self.e, self.e.x + self.e.xa, self.e.y + self.e.ya)

  -- dampen x

  -- walk friction
  -- gives a smooth accelerate and decelerate
  self.e.xa = self.e.xa * self.friction_walk

  -- zero small acceleration
  -- so entity does not appear to hop after a few seconds standing still
  if self.e.xa > -0.1 and self.e.xa < 0.1 then
    self.e.xa = 0
  end

  -- dampen y

  -- if you are falling you can't be jumping
  if self.e.ya > 0 then
    self.state_jumping = false
    self.state_double_jumping = false
  end

  -- gravity while jumping
  -- allows controllable jump height
  if self.state_jumping and self.e.ya < 0 and self.input_jump_hold then
    self.e.ya = self.e.ya + self.speed_gravity * self.friction_jump_gravity
  -- if not jumping and in the air apply normal gravity
  elseif not self.state_grounded then
    self.e.ya = self.e.ya + self.speed_gravity
  end

end


return platformer
