
--- Entity module.
-- Extends Component.
-- @classmod Entity


local setColor, rectangle = love.graphics.setColor, love.graphics.rectangle


local Entity = {}


--- New entity.
function Entity:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self
  o:init()
  return o
end


--- Init the entity.
function Entity:init()

  --- World in
  -- @field self.world
  -- @see World

  --- x position
  self.x = self.x or 0
  --- y position
  self.y = self.y or 0
  --- width
  self.w = self.w or 10
  --- height
  self.h = self.h or 10

  --- x acceleration
  -- zeroed after a collision
  self.xa = 0
  --- y acceleration
  -- zeroed after a collision
  self.ya = 0
  --- x velocity
  self.dx = 0
  --- y velocity
  self.dy = 0
  --- angle
  self.angle = 0
  --- types
  self.types = {}
  --- world zones in
  self.zones = {}
  --- world entity pool in
  self.pool = nil
  --- world
  self.collides = {}
end


--- Resolve an entity entity collision.
-- @tparam entity entity entity data table
-- @tparam entity e2 other entity
-- @tparam number nx new x position
-- @tparam number ny new y position
-- @tparam world.direction dir direction
function Entity:entityCollision(e2, nx, ny, dir)
  self:move(nx, ny)
  -- if dir == self.world.direction.U then
  --   self.y = e2.y + e2.h
  --   self.ya = 0
  -- elseif dir == self.world.direction.D then
  --   self.y = e2.y - self.h
  --   self.ya = 0
  -- elseif dir == self.world.direction.L then
  --   self.x = e2.x + e2.w
  --   self.xa = 0
  -- elseif dir == self.world.direction.R then
  --   self.x = e2.x - self.w
  --   self.xa = 0
  -- end
end


--- Resolve an entity tile collision.
-- @tparam Tile t tile
-- @tparam number nx x
-- @tparam number ny y
-- @tparam number nx new x
-- @tparam number nx new y
-- @tparam world.direction d direction the entity is moving in
function Entity:tileCollision(t, x, y, nx, ny, dir)
  if dir == self.world.direction.U then
    self.y = y * self.world.tileSize + self.world.tileSize
    self.ya = 0
  elseif dir == self.world.direction.D then
    self.y = y * self.world.tileSize - self.h
    self.ya = 0
  elseif dir == self.world.direction.L then
    self.x = x * self.world.tileSize + self.world.tileSize
    self.xa = 0
  elseif dir == self.world.direction.R then
    self.x = x * self.world.tileSize - self.w
    self.xa = 0
  end
end


--- Resolve an entity movement update if there was no collision.
-- @tparam entity entity entity data table
-- @tparam number nx new x
-- @tparam number ny new y
function Entity:move(nx, ny)
  self.x = nx
  self.y = ny
end


--- Update entity.
-- @number dt delta time
function Entity:update(dt)
end


--- Default entity draw function.
-- @tparam entity entity entity data table
function Entity:draw()
  setColor(255, 255, 255)
  rectangle('line', self.x, self.y, self.w, self.h)
end


return Entity
