
local vector = require('vector')
local Entity = require('entity')
local Keyboard = require('keyboard')
local Joystick = require('joystick')
local Sprite = require('sprite')

local Player = Entity:new()


function Player:init()
  self.soundDashHit = love.audio.newSource('sound/Hit 1.wav', 'static')
  self.soundSmash = love.audio.newSource('sound/smash.wav', 'static')
  self.soundSmash:setVolume(5)
  self.soundSword = love.audio.newSource('sound/sword swipe.wav', 'static')
  self.soundDash = love.audio.newSource('sound/like the wind.wav', 'static')

  -- super
  Entity.init(self)

  self.w = 16
  self.h = 16

  self.health = 120
  self.healthMax = 120

  self.color = { 255, 255, 255 }

  self.pull = 150

  self.score = 0

  if self.spriteTileset then
    self.sprite = Sprite:newSprite(self.spriteTileset, self)

    function self.sprite:endAnimation()
      if self.state == 'attack' then
        self.entity.state = 'stand'
        return false
      end
      return true
    end
  end

  self.state = 'stand'

  self.pullEntity = Entity:new({ x = self.x - self.pull / 2, y = self.y - self.pull / 2,
    w = self.pull, h = self.pull, player = self })
  self.pullEntity.types.pull = true

  if self.world then
    self.world:addEntity(self.pullEntity)
  end

  if self.attackTileset then

    self.attackEntity = Entity:new({ x = self.x, y = self.y, w = 16, h = 20, player = self })
    self.attackEntity.types.attack = true
    local attackEntity = self.attackEntity

    function attackEntity:entityCollision(e2, nx, ny, dir)

      if e2.types.creep and self.player.state == 'attack' then
        self.player.soundSword:play()
        e2.state = 'death'
      end

      if e2 ~= self.player then
        if e2.types.player and self.player.state == 'attack' then
          self.player.soundSword:play()
          e2:doDamageMelee(self, self.player.damageMelee, self.player.meleeTicks)
        end
      end

    end

    function attackEntity:update(dt)
      self.world:moveEntity(self, self.x, self.y)
    end

    self.world:addEntity(self.attackEntity)
    self.attackSprite = Sprite:newSprite(self.attackTileset, self)
  end

  self.types.player = true

  -- 1 == right, -1 == left
  self.d = 1

  self.walkSpeed = 1.8
  self.dashSpeed = 4
  self.speed = self.walkSpeed

  self.damageDash = 8
  self.damageMelee = 12

  self.meleeDamages = {}
  self.meleeTicks = 1

  self.dashes = {}
  self.dashVector = { x = 0, y = 0 }
  self.dashCount = 0
  self.dashDefault = 0.30
  self.dashTotal = self.dashDefault
  self.dashTime = 0
  self.dashComboTime = 0
  self.dashComboTotal = 1.8
  self.dashCombo = false
  self.dashAgainTime = 0
  self.dashAgainTotal = 0.8
  self.canDash = true
  self.dashTicks = 1

  self.ps = love.graphics.newParticleSystem(game:image('image/all/star.png'))
  self.ps:setSpread(21)
  self.ps:setSpeed(5, 20)
  self.ps:setPosition(0, 0)
  -- Colors as tables with optional alpha.
  self.ps:setColors(
    {255, 10, 10, 100},
    {255, 10, 10, 50}
  )
  -- Name changes ahead.
  self.ps:setEmitterLifetime(-1)
  self.ps:setParticleLifetime(0.5)

end


function Player:doDamageMelee(e, damage, tick)
  if self.meleeDamages[e] ~= tick then
    self.meleeDamages[e] = tick
    self.health = self.health - damage

    if self.health < 0 then
      if e.score then e.score = e.score + 1
        elseif e.player.score then e.player.score = e.player.score + 1 end

      self.health = self.healthTotal
      self.x = love.math.random(2, self.world.w - 2) * self.world.tileSize
      self.y = love.math.random(2, self.world.h - 2) * self.world.tileSize
    end
  end
end

function Player:dash()
  if self.soundDash:isPlaying() then
    self.soundDash:rewind()
  else
    self.soundDash:play()
  end

  self.dashes = { p1 = { x = self.x, y = self.y } }

  self.dashTicks = self.dashTicks + 1
  self.dashVector.x, self.dashVector.y = vector.unit(self.joystick.leftStick.x, self.joystick.leftStick.y)
  self.state = 'dash'
  self.speed = self.dashSpeed
  self.dashAgainTime = 0
  self.canDash = false
end


function Player:entityCollision(e2, nx, ny, dir)

  if self.state == 'dash' then
    if e2.types.player then
      e2:doDamageMelee(self, self.damageDash * (self.dashCount + 1), self.dashTicks)

      if self.dashCount > 0 then
        self.ps:setPosition(e2.x + e2.w/2, e2.y + e2.h/2)
        self.ps:emit(20)
        self.soundSmash:play()
        self.world.camera:shake(400, 60, 5)
      else
        self.soundDashHit:play()
      end

      self.dashCombo = false
      self.dashCount = 0
      self.dashComboTime = 0

    elseif e2.types.creep then
      e2.state = 'death'

      self.soundDashHit:play()

      self.dashHitCreep = true
      self.dashCombo = true
      self.dashComboTime = 0
    end
  end

  self.x = nx
  self.y = ny
  -- if dir == self.world.direction.U then
  --   self.y = e2.y + e2.h
  --   self.ya = 0
  -- elseif dir == self.world.direction.D then
  --   self.y = e2.y - self.h
  --   self.ya = 0
  -- elseif dir == self.world.direction.L then
  --   self.x = e2.x + e2.w
  --   self.xa = 0
  -- elseif dir == self.world.direction.R then
  --   self.x = e2.x - self.w
  --   self.xa = 0
  -- end
end


function Player:update(dt)
  self.ps:update(dt)
  self.joystick:update()

  if self.dashCombo then
    self.dashAgainTime = self.dashAgainTotal
    self.dashComboTime = self.dashComboTime + dt

    if self.dashComboTime > self.dashComboTotal then
      self.dashCombo = false
      self.canDash = true
      self.dashCount = 0
    end
  end

  if self.state == 'dash' then
    self.dashTime = self.dashTime + dt

    if self.dashTime > self.dashTotal then
      self.dashTime = 0
      self.state = 'stand'

      self.dashes.p2 = { x = self.x, y = self.y }

      if self.dashHitCreep then
        self.dashCount = self.dashCount + 1
      else
        self.dashCombo = false
      end

      self.dashHitCreep = false
    end
  else
    self.dashAgainTime = self.dashAgainTime + dt

    if self.dashAgainTime > self.dashAgainTotal then
      self.dashAgainTime = self.dashAgainTotal
      self.canDash = true
    end
  end

  if self.joystick.leftStick.x < - 0.3 then
    self.d = -1
  elseif self.joystick.leftStick.x > 0.3 then
    self.d = 1
  end

  self.dx = 0
  self.dy = 0

  if self.joystick then

    -- walk
    if self.joystick.rightTrigger > 0.3 and self.state ~= 'attack' then
      self.meleeTicks = self.meleeTicks + 1
      --self.world.camera:shake(400, 60, 3)
      self.state = 'attack'
      if self.attackTileset then
        self.attackSprite:setState('attack')
      end
    end

    -- dash
    if self.joystick.leftTrigger > 0.1 and (self.canDash) then
      self:dash()
    end

    -- moving
    if (math.abs(self.joystick.leftStick.x) > 0.2 or math.abs(self.joystick.leftStick.y) > 0.2)
      or self.state == 'dash' then

      if self.state == 'dash' then
        self.dx = self.dashVector.x * self.speed
        self.dy = self.dashVector.y * self.speed
      else
        self.dx = self.joystick.leftStick.x * self.speed
        self.dy = self.joystick.leftStick.y * self.speed
      end

      if self.state ~= 'walk' and self.state ~= 'attack' and self.state ~= 'dash' then
        self.state = 'walk'
        self.speed = self.walkSpeed
      end
    -- standing
    else
      if self.state ~= 'attack' then
        self.state = 'stand'
      end
    end
  end

  if self.sprite.state ~= self.state then
    self.sprite:setState(self.state)
  end

  self.world:moveEntity(self, self.x + self.dx, self.y + self.dy)

  self.sprite:update(dt)

  if self.attackTileset then
    self.attackSprite:update(dt)
    -- right
    if self.d == 1 then
      self.attackEntity.x = self.x + 4
      self.attackEntity.y = self.y - 4
    -- left
    else
      self.attackEntity.x = self.x - 4
      self.attackEntity.y = self.y - 4
    end
  end

    -- right
  if self.d == 1 then
    self.pullEntity.x = self.x - (self.pull - self.w) / 2
    self.pullEntity.y = self.y - (self.pull - self.h) / 2
  -- left
  else
    self.pullEntity.x = self.x - (self.pull - self.w) / 2
    self.pullEntity.y = self.y - (self.pull - self.h) / 2
  end
end


function Player:draw()
  local ox = 0
  if self.d == -1 then ox = self.w end

  love.graphics.setColor(255, 255, 255, 255)
  self.sprite.sheet.sb:add(self.sprite:getFrame(), self.x, self.y, 0, self.d, 1, ox, 0)

  if self.attackTileset and self.state == 'attack' then
    self.sprite.sheet.sb:add(self.attackSprite:getFrame(), self.x + (5 * self.d), self.y - 5, 0, self.d, 1, ox, 0)
  end
end


return Player
