
--- Game module.
-- @classmod Game

local Sheet = require('sheet')


local Game = {}


--- New game.
function Game:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self
  o:init()
  return o
end


function Game:init()
  --- all game screens
  self.screens = {}
  --- current game screen
  self.screen = nil
  -- game images
  self.images = {}
  -- game sprite sheets
  self.sheets = {}
  self.sounds = {}
end



--- Load, cache and return image.
-- Future calls for the same image will use cached version.
-- @tparam string path image path in the image folder
function Game:sound(path)
  if not self.sounds[path] then
    self.sounds[path] = love.audio.newSource(path, 'static')
  end
  return self.sounds[path]
end



--- Load, cache and return image.
-- Future calls for the same image will use cached version.
-- @tparam string path image path in the image folder
function Game:image(path)
  if not self.images[path] then
    self.images[path] = love.graphics.newImage(path)
  end
  return self.images[path]
end


--- Load, cache and return sheet.
-- Future calls for the same image will use cached version.
-- @tparam string path image path in the image folder
function Game:sheet(path)
  if not self.sheets[path] then
    self.sheets[path] = Sheet:new({ path = path })
  end
  return self.sheets[path]
end


function Game:addScreen(key, screen)
  self.screens[key] = screen
end


function Game:setScreen(key)
  self.key = key
  self.screen = self.screens[key]
end


function Game:update(dt)
  if self.screen then
    self.screen:update(dt)
  end
end


function Game:draw()
  if self.screen then
    self.screen:draw()
  else
    love.graphics.print('no screen', 5, 5)
  end
end


game = Game:new()


return game
