
--- Joystick module that allows keys to be rigged up to actions.
-- @classmod Joystick


local Joystick = {}


--- New joystick.
function Joystick:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self
  o:init()
  return o
end


--- Init the joystick.
function Joystick:init()
  assert(type(self.joystick) == 'userdata', 'required arg joystick must be a Love2D Joystick')

  --- Love2D Joystick object
  -- @field self.joystick

  --- current button state
  self.now = {}
  --- old button state
  self.old = {}
  --- left stick vector
  self.leftStick = { x = 0, y = 0 }
  --- right stick vector
  self.rightStick = { x = 0, y = 0 }
  --- left trigger
  self.leftTrigger = 0
  --- right trigger
  self.rightTrigger = 0
end


--- Update the current tick actions and remember the previous tick.
-- @tparam keyboard t keyboard data table
function Joystick:update()
  -- old buttons
  for k, v in pairs(self.now) do
    self.old[k] = v
  end

  -- now buttons
  self.now.up = self.joystick:isDown(0)
  self.now.down = self.joystick:isDown(1)
  self.now.left = self.joystick:isDown(2)
  self.now.right = self.joystick:isDown(3)
  self.now.starightTrigger = self.joystick:isDown(4)
  self.now.back = self.joystick:isDown(5)
  self.now.leftHat = self.joystick:isDown(6)
  self.now.rightHat = self.joystick:isDown(7)
  self.now.leftBumper = self.joystick:isDown(8)
  self.now.rightBumper = self.joystick:isDown(9)
  self.now.a = self.joystick:isDown(10)
  self.now.b = self.joystick:isDown(11)
  self.now.x = self.joystick:isDown(12)
  self.now.y = self.joystick:isDown(13)
  self.now.guide = self.joystick:isDown(14)

  -- axis
  self.leftStick.x = self.joystick:getAxis(1)
  self.leftStick.y = self.joystick:getAxis(2)
  self.rightStick.x = self.joystick:getAxis(3)
  self.rightStick.y = self.joystick:getAxis(4)
  self.leftTrigger = self.joystick:getAxis(5)
  self.rightTrigger = self.joystick:getAxis(6)
end


return Joystick
