
--- Keyboard module that allows keys to be rigged up to actions.
-- @classmod Keyboard


local isDown = love.keyboard.isDown


local Keyboard = {}


function Keyboard:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self
  o:init()
  return o
end


--- Init the keyboard.
function Keyboard:init()
  --- current action state
  self.now = {}
  --- old action state
  self.old = {}
  --- action keys
  self.keys = {}
end


--- Add an action name and the Love2D KeyConstant that triggers it.
-- @tparam string name the action name
-- @tparam string key the Love2D KeyConstant
function Keyboard:action(name, key)
  self.keys[name] = key
  self.now[name] = false
end


--- Update the current tick actions and remember the previous tick.
function Keyboard:update()
  -- copy now to old
  for k, v in pairs(self.now) do
    self.old[k] = v
  end

  -- go through keys and set now
  for k, v in pairs(self.keys) do
    self.now[k] = isDown(v)
  end
end


return Keyboard
