
local Keyboard = require('keyboard')
local Camera = require('camera')


local Intro = {}


--- New menu screen.
function Intro:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self
  o:init()
  return o
end


function Intro:init()
  self.music = love.audio.newSource('music/intro.mp3')
  self.music:setVolume(4)

  self.keyboard = Keyboard:new()
  self.keyboard:action('escape', 'escape')

  self.introImage = game:image('image/introduction.png')
  self.titleImage = game:image('image/title.png')

  self.zoom = 3

  self.camera = Camera:new({
    x = 0,
    y = 0,
    w = love.graphics.getWidth(),
    h = love.graphics.getHeight()})
  self.camera.zoom = 4

  self.camera.x = -(love.graphics.getWidth() / self.camera.zoom - 128)
  self.camera.y = -(love.graphics.getHeight() / self.camera.zoom - 128) / 2

  self.time = 0
  -- self.fadeinTime = 7
  self.fadeinTime = 7

  -- self.scrollTime = 24
  self.scrollTime = 28

  self.titleTime = 3

  self.state = 'fadein'
end


function Intro:update(dt)

  self.keyboard:update()

  self.time = self.time + dt

  if self.time > self.fadeinTime and self.state == 'fadein' then
    self.time = 0
    self.state = 'scroll'
  end

  if self.state == 'scroll' then
    self.camera.x = self.camera.x + 30 * dt
  end

  if self.time > self.scrollTime and self.state == 'scroll' then
    self.time = 0
    self.state = 'title'
    self.camera.x = -(love.graphics.getWidth() / self.camera.zoom - 128) / 2
  end

  local joysticks = love.joystick.getJoysticks()
  for i, joystick in ipairs(joysticks) do
    if joystick:isDown(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14) then
      self.play = true
    end
  end

end


function Intro:draw()
  self.camera:push()

  if self.state == 'fadein' then
    love.graphics.setColor(255, 255, 255, 255 * self.time / self.fadeinTime)
  elseif self.state == 'title' then
    love.graphics.setColor(255, 255, 255, 255 * self.time / self.titleTime)
  else
    love.graphics.setColor(255, 255, 255, 255)
  end

  if self.state == 'title' then
    love.graphics.draw(self.titleImage, 0, 0)
  else
    love.graphics.draw(self.introImage, 0, 0)
  end
  self.camera:pop()


  if self.keyboard.now.escape then love.event.quit() end

  if self.play then
    self.music:stop()
    screen = play
    play.music:play()
  end
end


return Intro
