
local vector = require('vector')
local Camera = require('camera')
local Entity = require('entity')
local Sprite = require('sprite')
local Keyboard = require('keyboard')
local Joystick = require('joystick')
local World = require('world')


local Play = {}


--- New menu screen.
function Play:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self
  o:init()
  return o
end


function Play:init()
  self.music = love.audio.newSource('music/fight.mp3')
  self.music:setVolume(0.3)

  self.keyboard = Keyboard:new()
  self.keyboard:action('escape', 'escape')

  local tileSize = 16
  local zoom = 3

  local w = math.ceil(love.graphics.getWidth() / zoom / tileSize)
  local h = math.ceil(love.graphics.getHeight() / zoom / tileSize)
  local x = (love.graphics.getWidth() - (w * zoom * tileSize)) / zoom / 2
  local y = (love.graphics.getHeight() - (h * zoom * tileSize)) / zoom / 2

  self.camera = Camera:new({ x = -x, y = -y, w = love.graphics.getWidth(), h = love.graphics.getHeight()})
  self.camera.zoom = zoom
  local camera = self.camera

  function love.resize(w, h)
    camera.w = w
    camera.h = h
  end

  self.world = World:newEmpty(self, w, h, tileSize)

  --self.world = JamWorld:newMap('1', self.camera)
end


function Play:update(dt)
  self.keyboard:update()

  if self.keyboard.now.escape then love.event.quit() end

  self.world:update(dt)
  self.camera:update(dt)
end


function Play:draw()
  love.graphics.setColor(255, 255, 255, 255)
  self.world:draw()
  --self.world:drawDebug()

  --love.graphics.print(love.timer.getFPS(), 5, 5)

  -- ui
  if self.player1 then

    love.graphics.setColor(200, 10, 10, 255)
    love.graphics.rectangle('fill', 2 * self.camera.zoom, 2 * self.camera.zoom,
      math.max(self.player1.health * self.camera.zoom, 0), 6 * self.camera.zoom)

    love.graphics.setColor(10, 200, 10, 255)
    love.graphics.rectangle('fill', 2 * self.camera.zoom, 8 * self.camera.zoom,
      self.player1.dashAgainTime / self.player1.dashAgainTotal * self.camera.zoom * self.player1.healthMax, 6 * self.camera.zoom)

    local w = (self.player1.healthMax * self.camera.zoom) - self.player1.dashComboTime / self.player1.dashComboTotal * self.camera.zoom * self.player1.healthMax

    love.graphics.setColor(10, 10, 200, 255)
    love.graphics.rectangle('fill', 2 * self.camera.zoom, 14 * self.camera.zoom,
      math.max(w, 0), 6 * self.camera.zoom)

    love.graphics.setColor(40, 40, 40, 255)
    love.graphics.rectangle('line', 2 * self.camera.zoom, 2 * self.camera.zoom,
      self.player1.healthMax * self.camera.zoom, 18 * self.camera.zoom)

    love.graphics.setColor(0, 0, 0, 255)
    love.graphics.print(self.player1.score, 2 * self.camera.zoom, 20 * self.camera.zoom, 0, 3, 3)

    love.graphics.setColor(0, 0, 0, 255)
    love.graphics.print(self.player1.dashCount, 2 * self.camera.zoom, 30 * self.camera.zoom, 0, 3, 3)

  end


  if self.player2 then
    local p2w = self.player2.health * self.camera.zoom
    local x = love.graphics.getWidth() - 2 * self.camera.zoom - self.player2.healthMax * self.camera.zoom

    love.graphics.setColor(200, 10, 10, 255)
    love.graphics.rectangle('fill', x, 2 * self.camera.zoom,
      math.max(p2w, 0), 6 * self.camera.zoom)

    love.graphics.setColor(10, 200, 10, 255)
    love.graphics.rectangle('fill', x, 8 * self.camera.zoom,
      self.player2.dashAgainTime / self.player2.dashAgainTotal * self.camera.zoom * self.player2.healthMax, 6 * self.camera.zoom)

    local w = (self.player2.healthMax * self.camera.zoom) - self.player2.dashComboTime / self.player2.dashComboTotal * self.camera.zoom * self.player2.healthMax

    love.graphics.setColor(10, 10, 200, 255)
    love.graphics.rectangle('fill', x, 14 * self.camera.zoom,
      math.max(w, 0), 6 * self.camera.zoom)

    love.graphics.setColor(40, 40, 40, 255)
    love.graphics.rectangle('line', x, 2 * self.camera.zoom, self.player2.healthMax * self.camera.zoom, 18 * self.camera.zoom)

    love.graphics.setColor(0, 0, 0, 255)
    love.graphics.print(self.player2.score, x, 20 * self.camera.zoom, 0, 3, 3)

    love.graphics.setColor(0, 0, 0, 255)
    love.graphics.print(self.player2.dashCount, x, 30 * self.camera.zoom, 0, 3, 3)
  end

end


return Play
