
local newQuad = love.graphics.newQuad


local Sheet = {}


--- Create a new sheet.
-- @tparam[opt] table o
-- @usage local sheet = Sheet:new()
function Sheet:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self
  o:init()
  return o
end


--- Sheet init function automatically called by new function.
-- Init the sprite batch, master and sub images.
function Sheet:init()
  assert(type(self.path) == 'string', 'required arg path must be a string')

  local mapping = require('image')

  self.image = game:image(self.path .. '.png')
  self.w = self.image:getWidth()
  self.h = self.image:getHeight()
  self.sb = love.graphics.newSpriteBatch(self.image)
  self.parts = {}

  local data = require(self.path)

  for k, s in pairs(data) do

    self.parts[k] = {
      quad = newQuad(s.x, s.y, s.w, s.h, self.w, self.h),
    }

    if mapping[k] then
      self.parts[k].quads = {}

      local mw = mapping[k].w
      local mh = mapping[k].h
      local w = math.floor(s.w / mw)
      local h = math.floor(s.h / mh)

      for x = 0, w - 1 do
        self.parts[k].quads[x] = {}
        for y = 0, h - 1 do
          self.parts[k].quads[x][y] = newQuad(s.x + x * mw, s.y + y * mh, mw, mh, self.w, self.h)
        end
      end
    end
  end
end


return Sheet
