
-- Sprite table that supports multiple animation states.
local Sprite = {}


--- Create a new sprite.
-- @tparam[opt] table o
-- @usage local sprite = Sprite:new()
function Sprite:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self
  o:init()
  return o
end


--- empty init
function Sprite:init()
  --- animations and frame quads
  self.animations = {}
  -- current animation state
  self.state = nil
  -- current animation frame
  self.frame = 1
  -- current ticks
  self.time = 0
end


function Sprite:newSprite(name, entity)
  -- hardcoded sprite sheet
  local sheet = game:sheet('image/all')
  local data = require('sprite.' .. name)

  local sprite = self:new({ sheet = sheet, entity = entity })
  sprite.data = data
  sprite.name = name

  local count = 0

  for state, animation in pairs(data.animations) do
    count = count + 1
    sprite.animations[state] = {}
    for i, frame in ipairs(animation) do
      sprite.animations[state][i] = {}
      sprite.animations[state][i].quad = sheet.parts[data.sheet].quads[frame[1]][frame[2]]
      sprite.animations[state][i].time = frame[3]
    end
  end

  -- if only one animation set it
  if count == 1 then
    for state, animation in pairs(data.animations) do
      sprite:setState(state)
    end
  end

  return sprite
end


function Sprite:setState(state)
  self.state = state
  self.frame = 1
  self.time = 0
end


function Sprite:endAnimation()
  return true
end


function Sprite:update(dt)
  self.time = self.time + dt

  if(self.time > self.animations[self.state][self.frame].time) then
    self.time = self.time - self.animations[self.state][self.frame].time
    self.frame = self.frame + 1

    if self.frame > #self.animations[self.state] then
      self.frame = 1
      self:endAnimation()
    end
  end
end


function Sprite:getFrame()
  return self.animations[self.state][self.frame].quad
end


return Sprite
